#include "stdafx.h"
#define INITGUID
#include <initguid.h>
#include "poom.h"

#ifndef RELEASE_OBJ
#define RELEASE_OBJ(s)  \
    if (s != NULL)      \
    {                   \
        s->Release();   \
        s = NULL;       \
    }
#endif //RELEASE_OBJ

using namespace POOM;

CApp::CApp(void) : m_pApp(NULL) {
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if (hr == S_FALSE) {
		//ɏ
		m_AlreadyInitialized = TRUE;
	} else if (FAILED(hr)) {
		//G[
		return;
	}

	hr = CoCreateInstance(
		CLSID_Application,
		NULL,
		CLSCTX_INPROC_SERVER,
		IID_IPOutlookApp,
		(LPVOID*)&m_pApp);
	if (FAILED(hr)) return;

	hr = m_pApp->Logon(NULL);
	if (FAILED(hr)) {
		m_pApp->Release();
		m_pApp = NULL;
		return;
	}
}

CApp::~CApp(void) {
	if (m_pApp) {
		m_pApp->Logoff();
		m_pApp->Release();
	}

	if (!m_AlreadyInitialized) {
		CoUninitialize();
	}
}

IPOutlookApp &CApp::GetApp() {
	return *m_pApp;
}

bool CApp::CreateAppointment(const ITEM& item, bool IsReminderSet) {
	if (!m_pApp) {
		return false;
	}

	IAppointment *pItem = NULL;
	HRESULT hr = m_pApp->CreateItem(olAppointmentItem, (IDispatch**)&pItem);
	if (FAILED(hr)) return false;

	//
	BSTR bSubject = SysAllocString(item.Subject.c_str());
	pItem->put_Subject(bSubject);
	SysFreeString(bSubject);

	//ꏊ
	BSTR bLocation = SysAllocString(item.Location.c_str());
	pItem->put_Location(bLocation);
	SysFreeString(bLocation);

	//Jn
	DATE dStart;
	m_pApp->SystemTimeToVariantTime((LPSYSTEMTIME)&item.StartTime, &dStart);
	pItem->put_Start(dStart);

	//I
	DATE dEnd;
	m_pApp->SystemTimeToVariantTime((LPSYSTEMTIME)&item.EndTime, &dEnd);
	if (item.IsAllDayEvent) dEnd++;
	pItem->put_End(dEnd);

	//I\
	pItem->put_AllDayEvent(item.IsAllDayEvent);

	//A[(onȂ15O߂)
	pItem->put_ReminderSet(IsReminderSet);
	if (IsReminderSet) {
		pItem->put_ReminderMinutesBeforeStart(15);
	}

	//ۑ
	pItem->Save();
	pItem->Release();

	return true;
}

bool CApp::CreateTask(const ITEM& item, bool IsReminderSet) {
	if (!m_pApp) {
		return false;
	}

	ITask *pItem = NULL;
	HRESULT hr = m_pApp->CreateItem(olTaskItem, (IDispatch**)&pItem);
	if (FAILED(hr)) return false;

	//
	BSTR bSubject = SysAllocString(item.Subject.c_str());
	pItem->put_Subject(bSubject);
	SysFreeString(bSubject);

	//Jn
	if (!IsEmptyTime(&item.StartTime)) {
		DATE dStart;
		m_pApp->SystemTimeToVariantTime((LPSYSTEMTIME)&item.StartTime, &dStart);
		pItem->put_StartDate(dStart);
	}

	//
	if (!IsEmptyTime(&item.EndTime)) {
		DATE dEnd;
		m_pApp->SystemTimeToVariantTime((LPSYSTEMTIME)&item.EndTime, &dEnd);
		pItem->put_DueDate(dEnd);
	} else {
		//̂ŃA[Ȃɂ
		IsReminderSet = false;
	}

	//A[(onȂ8:00߂)
	pItem->put_ReminderSet(IsReminderSet);
	if (IsReminderSet) {
		SYSTEMTIME reminderTime;
		DATE dReminder;
		CopyMemory(&reminderTime, &item.EndTime, sizeof(SYSTEMTIME));
		reminderTime.wHour = 8;
		reminderTime.wMinute = 0;
		m_pApp->SystemTimeToVariantTime((LPSYSTEMTIME)&reminderTime, &dReminder);
		pItem->put_ReminderTime(dReminder);
	}

	//ۑ
	pItem->Save();
	pItem->Release();

	return true;
}

bool CApp::IsEmptyTime(const SYSTEMTIME *lpTime) {
	SYSTEMTIME st;
	ZeroMemory(&st, sizeof(SYSTEMTIME));
	return (memcmp(lpTime, &st, sizeof(SYSTEMTIME)) == 0);
}
